import java.util.List;

import org.jbpm.api.ProcessInstance;
import org.jbpm.api.history.HistoryTask;
import org.jbpm.api.job.Job;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;


/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @author Sebastian Schneider
 */
public class TaskTimeoutTest extends JbpmTestCase {
  
 String deploymentId;
  
  protected void setUp() throws Exception {
    super.setUp();
    
    deploymentId = repositoryService.createDeployment()
        .addResourceFromClasspath("process.jpdl.xml")
        .deploy();
  }

  protected void tearDown() throws Exception {
    repositoryService.deleteDeploymentCascade(deploymentId);
    
    super.tearDown();
  }

  public void testTaskTimeout() {
    ProcessInstance processInstance = executionService.startProcessInstanceByKey("TaskTimeout");
    String processInstanceId = processInstance.getId();
    
    List<Task> tasks = taskService.findPersonalTasks("alex");
    assertEquals(1,tasks.size());
    
    Job job = managementService.createJobQuery()
    .timers()
    .processInstanceId(processInstance.getId())
    .uniqueResult();
  
    managementService.executeJob(job.getId());
    
    tasks = taskService.findPersonalTasks("alex");
    assertEquals(0, tasks.size());
    
    assertEquals(1,historyService.createHistoryTaskQuery().assignee("alex").count());
    HistoryTask historyTask = historyService.createHistoryTaskQuery().assignee("alex").uniqueResult();
    assertEquals("timed out", historyTask.getState());
      
    processInstance = executionService.findProcessInstanceById(processInstanceId);  
    assertTrue(processInstance.isActive("timed out"));
  }
  
  public void testNoTaskTimeout() {
    ProcessInstance processInstance = executionService.startProcessInstanceByKey("TaskTimeout");
    String processInstanceId = processInstance.getId();
    
    List<Task> tasks = taskService.findPersonalTasks("alex");
    assertEquals(1,tasks.size());
    
    taskService.completeTask(tasks.get(0).getId(),"to completed");
    processInstance = executionService.findProcessInstanceById(processInstanceId);
    
    assertTrue(processInstance.isActive("completed"));
    
    tasks = taskService.findPersonalTasks("alex");
    assertEquals(0, tasks.size());
    
    assertEquals(1,historyService.createHistoryTaskQuery().assignee("alex").count());
    HistoryTask historyTask = historyService.createHistoryTaskQuery().assignee("alex").uniqueResult();
    assertEquals("completed", historyTask.getState());
    
    processInstance = executionService.findProcessInstanceById(processInstanceId);  
    assertTrue(processInstance.isActive("completed"));
  }
}
